import os
from pathlib import Path

def rename_pdf_files():
    # Check if PyPDF2 is installed
    try:
        import PyPDF2
    except ImportError:
        print("PyPDF2 is not installed. Please install it with 'pip install PyPDF2'")
        return

    # Identify the current working directory
    directory = os.getcwd()

    # Determine max title length based on current directory length
    max_title_length = 260 - len(directory) - 1  # Subtract 1 for the slash between directory and filename

    # Enumerate all files in the directory
    for filename in os.listdir(directory):
        # Only process PDF files
        if filename.endswith(".pdf"):
            try:
                # Open the PDF file
                with open(os.path.join(directory, filename), "rb") as file:
                    reader = PyPDF2.PdfReader(file)

                    # Get the document info
                    info = reader.metadata

                    # Extract the title from the metadata
                    metadata_title = info.get('/Title', '')

                    # Remove forbidden characters in filenames
                    forbidden_chars = ['<', '>', ':', '"', '/', '\\', '|', '?', '*']
                    for char in forbidden_chars:
                        metadata_title = metadata_title.replace(char, '')

                    # Limit to max title length
                    metadata_title = metadata_title[:max_title_length]

                # Make sure the file is closed before renaming
                if metadata_title:
                    new_filename = f"{directory}/{metadata_title}.pdf"
                    if os.path.exists(new_filename):
                        print(f"A file with the name '{metadata_title}.pdf' already exists. Skipping this file.")
                    else:
                        Path(os.path.join(directory, filename)).rename(new_filename)
                        print(f"Renamed '{filename}' to '{metadata_title}.pdf'")
                else:
                    print(f"No title found in the metadata of '{filename}'. File name was not changed.")
            except PyPDF2.errors.PdfReadError:
                print(f"Could not read '{filename}'. Skipping this file.")
            except PermissionError:
                print(f"No permission to rename '{filename}'. Skipping this file.")
            except Exception as e:
                print(f"An error occurred while processing '{filename}': {str(e)}")

# Call the function
rename_pdf_files()

# Pause before exiting
input("Press enter to exit...")